@@ -68,7 +68,6 @@ sub get_requests {
     my $self = shift;
     my $jmx = shift;
     my $args = shift;
-
     # If a script is given, extract a subref and return it
     return [ $self->_extract_script_as_subref($jmx) ] if $self->script;
 
@@ -94,14 +93,20 @@ sub get_requests {
     }
     push @requests,$request;
 
-    if ($self->base) {
+    if ($self->base || $self->base_mbean) {
         if (!looks_like_number($self->base)) {
             # It looks like a number, so we will use the base literally
-            my $alias = JMX::Jmx4Perl::Alias->by_name($self->base);
+            my $alias;
+            
+            if ($self->base) {
+                $alias = JMX::Jmx4Perl::Alias->by_name($self->base);
+            }
             if ($alias) {
                 push @requests,new JMX::Jmx4Perl::Request(READ,$jmx->resolve_alias($self->base));
             } else {
-                my ($mbean,$attr,$path) = $self->_split_attr_spec($self->base);
+                my ($mbean,$attr,$path) = $self->base_mbean ? 
+                  ($self->base_mbean, $self->base_attribute, $self->base_path) : 
+                    $self->_split_attr_spec($self->base);
                 die "No MBean given in base name ",$self->base unless $mbean;
                 die "No Attribute given in base name ",$self->base unless $attr;
                 
@@ -187,19 +192,26 @@ sub extract_responses {
     # Normalize value 
     my ($value_conv,$unit) = $self->_normalize_value($value);
     my $label = $self->_get_name(cleanup => 1);
-    if ($self->base && !$script_mode) {
+    if ( ($self->base || $self->base_mbean) && !$script_mode) {
         # Calc relative value 
         my $base_value = $self->_base_value($self->base,$responses,$requests);
         my $rel_value = sprintf "%2.2f",$base_value ? (int((($value / $base_value) * 10000) + 0.5) / 100) : 0;
                 
         # Performance data. Convert to absolute values before
-        my ($critical,$warning) = $self->_convert_relative_to_absolute($base_value,$self->critical,$self->warning);
-        $np->add_perfdata(label => $label,value => $value,
-                          critical => $critical,warning => $warning,
-                          min => 0,max => $base_value,
-                          $self->unit ? (uom => $self->unit) : ());
+        if ($self->_include_perf_data) {
+            if ($self->perfdata && $self->perfdata =~ /^\s*\%\s*/) {
+                $np->add_perfdata(label => $label, value => $rel_value, uom => '%',
+                                  critical => $self->critical, warning => $self->warning);
+            } else {
+                my ($critical,$warning) = $self->_convert_relative_to_absolute($base_value,$self->critical,$self->warning);
+                $np->add_perfdata(label => $label,value => $value,
+                                  critical => $critical,warning => $warning,
+                                  min => 0,max => $base_value,
+                                  $self->unit ? (uom => $self->unit) : ());
+            }
+        }
         # Do the real check.
-        my ($code,$mode) = $self->_check_threshhold($rel_value);
+        my ($code,$mode) = $self->_check_threshold($rel_value);
         # For Multichecks, we remember the label of a currently failed check
         $self->_update_error_stats($opts->{error_stat},$code) unless $code == OK;
         my ($base_conv,$base_unit) = $self->_normalize_value($base_value);
@@ -208,12 +220,15 @@ sub extract_responses {
                                                     base_unit => $base_unit, prefix => $opts->{prefix}));            
     } else {
         # Performance data
-        $np->add_perfdata(label => $label,
-                          critical => $self->critical, warning => $self->warning, 
-                          value => $value,$self->unit ? (uom => $self->unit) : ());
+        $value = $self->_sanitize_value($value);
+        if ($self->_include_perf_data) {
+            $np->add_perfdata(label => $label,
+                              critical => $self->critical, warning => $self->warning, 
+                              value => $value,$self->unit ? (uom => $self->unit) : ());
+        }
         
         # Do the real check.
-        my ($code,$mode) = $self->_check_threshhold($value);
+        my ($code,$mode) = $self->_check_threshold($value);
         $self->_update_error_stats($opts->{error_stat},$code) unless $code == OK;
         $np->add_message($code,$self->_exit_message(code => $code,mode => $mode,value => $value_conv, unit => $unit,
                                                     prefix => $opts->{prefix}));                    
@@ -221,12 +236,22 @@ sub extract_responses {
     return @extra_requests;
 }
 
+sub _include_perf_data {
+    my $self = shift;
+    # No perf dara for string based checks by default
+    my $default = not defined($self->string);
+    # If 'PerfData' is set explicitely to false/off/no/0 then no perfdata
+    # will be included
+    return $default unless defined($self->perfdata);
+    return $self->perfdata !~ /^\s*(false|off|no|0)\s*$/i;
+}
+
 sub _update_error_stats {
     my $self = shift;
     my $error_stat = shift || return;
     my $code = shift;
 
-    my $label = $self->{config}->{key} || $self->{config}->{name};
+    my $label = $self->{config}->{name} || $self->{config}->{key};
     if ($label) {
         my $arr = $error_stat->{$code} || [];
         push @$arr,$label;
@@ -339,6 +364,7 @@ sub _base_value {
     my $resp = shift @{$responses};
     my $req = shift @{$requests};
     $self->nagios_die($resp->{error}) if $resp->{error};
+    #print Dumper($req,$resp);
     return $self->_extract_value($req,$resp);
 }
 
@@ -403,6 +429,14 @@ sub _normalize_value {
     die "Unknown unit '$unit' for value $value";
 }
 
+sub _sanitize_value {
+    my ($self,$value) = @_;
+    if ($value =~ /\de/i) {
+        $value = sprintf("%f", $value);
+    }
+    return $value;
+}
+
 sub _verify_response {
     my ($self,$req,$resp) = @_;
     my $np = $self->{np};
@@ -434,7 +468,7 @@ sub _get_name {
             } else {
                 my $val = $self->value;
                 if ($val) {
-                    $name = "[" . $self->value . "]";
+                    $name = "[" . $val . "]";
                 } else {
                     my $a_or_o = $self->attribute || $self->operation || "";
                     my $p = $self->path ? "," . $self->path : "";
@@ -515,10 +549,10 @@ sub _split_attr_spec {
         $p =~ s|\\(.)|$1|sg;
         push @ret,$p;
     }    
-    return @ret;
+    return (shift(@ret),shift(@ret),join("/",@ret));
 }
 
-sub _check_threshhold {
+sub _check_threshold {
     my $self = shift;
     my $value = shift;
     my $np = $self->{np};
@@ -535,7 +569,7 @@ sub _check_threshhold {
            defined($self->critical) ? (critical => $self->critical) : (),
            defined($self->warning) ? (warning => $self->warning) : ()
           );            
-        return ($np->check_threshold(check => $value,@ths),"numeric");    
+        return (@ths ? $np->check_threshold(check => $value,@ths) : OK,"numeric");    
     } else {
         return
           ($self->_check_string_threshold($value,CRITICAL,$self->critical) ||
@@ -581,7 +615,7 @@ sub _exit_message {
     my $code = $args->{code};
     my $mode = $args->{mode};
     if ($code == CRITICAL || $code == WARNING) {
-        if ($self->base) {
+        if ($self->base || $self->base_mbean) {
             return $self->_format_label
               ('%n : Threshold \'%t\' failed for value %.2r% ('. &_placeholder($args,"v") .' %u / '.
                &_placeholder($args,"b") . ' %u)',$args);
@@ -594,7 +628,7 @@ sub _exit_message {
             }
         }
     } else {
-        if ($self->base) {
+        if ($self->base || $self->base_mbean) {
             return $self->_format_label('%n : In range %.2r% ('. &_placeholder($args,"v") .' %u / '.
                                         &_placeholder($args,"b") . ' %w)',$args);
         } else {
@@ -630,6 +664,7 @@ sub _format_label {
     # %t : threshold failed ("" for OK or UNKNOWN)
     # %c : code ("OK", "WARNING", "CRITICAL", "UNKNOWN")
     # %d : delta
+    # 
     my @parts = split /(\%[\w\.\-]*\w)/,$label;
     my $ret = "";
     foreach my $p (@parts) {
@@ -645,11 +680,7 @@ sub _format_label {
             } elsif ($what eq "f") {
                 $ret .= sprintf $format . "f",$args->{value};
             } elsif ($what eq "v") {
-                if ($args->{mode} ne "numeric") {
-                    $ret .= sprintf $format . "s",$args->{value};
-                } else {
-                    $ret .= sprintf $format . &_format_char($args->{value}),$args->{value};
-                }
+                $ret .= &_format_value($format,$args->{mode},$args->{value});
             } elsif ($what eq "t") {
                 my $code = $args->{code};
                 my $val = $code == CRITICAL ? $self->critical : ($code == WARNING ? $self->warning : "");
@@ -660,6 +691,10 @@ sub _format_label {
                 $ret .= sprintf $format . "s",$self->_get_name();
             } elsif ($what eq "d") {
                 $ret .= sprintf $format . "d",$self->delta;
+            } elsif ($what eq "y") {
+                $ret .= &_format_value($format,$args->{mode},$self->warning);
+            } elsif ($what eq "z") {
+                $ret .= &_format_value($format,$args->{mode},$self->critical);                
             }
         } else {
             $ret .= $p;
@@ -674,6 +709,16 @@ sub _format_label {
     }
 }
 
+sub _format_value {
+    my $format = shift;
+    my $mode = shift;
+    my $value = shift;
+    if ($mode ne "numeric") {
+        return sprintf $format . "s",$value;
+    } else {
+        return sprintf $format . &_format_char($value),$value;
+    }    
+}
 sub _format_char {
     my $val = shift;
     $val =~ /\./ ? "f" : "d";
@@ -697,14 +742,16 @@ my $CHECK_CONFIG_KEYS = {
                          "delta" => "delta",
                          "name" => "name",
                          "base" => "base",
+                         "base-mbean" => "basembean",
+                         "base-attribute" => "baseattribute",
+                         "base-path" => "basepath",
                          "unit" => "unit",
                          "numeric" => "numeric",
                          "string" => "string",
                          "label" => "label",
-
+                         "perfdata" => "perfdata",
                          "value" => "value",
                          "null" => "null",
-
                          "script" => "script"
                         };
 
@@ -716,7 +763,7 @@ sub AUTOLOAD {
     my $np = $self->{np};
     my $name = $AUTOLOAD;
     $name =~ s/.*://;   # strip fully-qualified portion
-    $name =~ s/-/_/g;
+    $name =~ s/_/-/g;
 
     if ($CHECK_CONFIG_KEYS->{$name}) {
         return $np->opts->{$name} if defined($np->opts->{$name});
